#define _POSIX_C_SOURCE (199309L)

#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>

#define REPETITIONS (100)

typedef struct {
    char name[32U];
    float value;
} sensor_value_t;

static void delay(void)
{
    struct timespec waiting_time = { 0, rand() };
    nanosleep( &waiting_time, NULL);
}

static void sensor_runner(sensor_value_t *entry, char *name, float value)
{
    printf("Run %s sensor\n", name);
    for (size_t i = 0; i < REPETITIONS; ++i)
    {
        snprintf(entry->name, sizeof(entry->name), "%s", name);
        delay();
        entry->value = value;
        delay();
    }
}

static void *temperature_sensor(void *arg)
{
    sensor_runner((sensor_value_t*) arg, "Temperature", 23.4567F);
    return NULL;
}

static void *humditity_sensor(void *arg)
{
    sensor_runner((sensor_value_t*) arg, "Humidity", 62.12F);
    return NULL;
}

static void *db_writer(void *arg)
{
    sensor_value_t *entry = (sensor_value_t*) arg;
    for (size_t i = 0; i < REPETITIONS; ++i)
    {
        printf("Read sensor value:" \
               "\tSensor:\t%s" \
               "\tValue:\t%3.4f\n",
               entry->name, entry->value);
        delay();
    }
    return NULL;
}

int main(void)
{
    srand(time(NULL));
    sensor_value_t database_entry;
    puts("Start threads");

    pthread_t sensor1, sensor2, databasewriter;

    pthread_create(&sensor1, NULL, temperature_sensor, &database_entry);
    pthread_create(&sensor2, NULL, humditity_sensor, &database_entry);
    pthread_create(&databasewriter, NULL, db_writer, &database_entry);

    pthread_join(sensor1, NULL);
    pthread_join(sensor2, NULL);
    pthread_join(databasewriter, NULL);
    return EXIT_SUCCESS;
}
